@@ -28,17 +28,71 @@
 
 #ifndef __NSObjCRuntime_h_GNUSTEP_BASE_INCLUDE
 #define __NSObjCRuntime_h_GNUSTEP_BASE_INCLUDE
+
+#include <stdarg.h>
+#include <limits.h>
+#include <float.h>
+
 #import	<GNUstepBase/GSVersionMacros.h>
-#import	<GNUstepBase/preface.h>
 #import	<GNUstepBase/GSConfig.h>
-#import	<GNUstepBase/GSObjCRuntime.h>
+
+/* These typedefs must be in place before GSObjCRuntime.h is imported.
+ */
+
+#if     !defined(NSINTEGER_DEFINED)
+typedef	intptr_t	NSInteger;
+typedef	uintptr_t	NSUInteger;
+#	define NSIntegerMax  INTPTR_MAX
+#	define NSIntegerMin  INTPTR_MIN
+#	define NSUIntegerMax UINTPTR_MAX
+#endif /* !defined(NSINTEGER_DEFINED) */
+
+#if     !defined(CGFLOAT_DEFINED)
+#if     GS_SIZEOF_VOIDP == 8
+#define CGFLOAT_IS_DBL  1
+typedef double          CGFloat;
+#define CGFLOAT_MIN	DBL_MIN
+#define CGFLOAT_MAX	DBL_MAX
+#else
+typedef float           CGFloat;
+#define CGFLOAT_MIN	FLT_MIN
+#define CGFLOAT_MAX	FLT_MAX
+#endif
+#endif /* !defined(CGFLOAT_DEFINED) */
+
+#define NSINTEGER_DEFINED 1
+#define CGFLOAT_DEFINED 1
+#ifndef NS_AUTOMATED_REFCOUNT_UNAVAILABLE
+#  if __has_feature(objc_arc)
+#    define NS_AUTOMATED_REFCOUNT_UNAVAILABLE \
+      __attribute__((unavailable("Not available with automatic reference counting")))
+#  else
+#    define NS_AUTOMATED_REFCOUNT_UNAVAILABLE
+#  endif
+#endif
+
 
 #if	defined(__cplusplus)
 extern "C" {
 #endif
 
-typedef	gsaddr	NSInteger;
-typedef	gsuaddr	NSUInteger;
+enum 
+{
+  NSEnumerationConcurrent = (1UL << 0), /** Specifies that the enumeration
+   * is concurrency-safe.  Note that this does not mean that it will be
+   * carried out in a concurrent manner, only that it can be.
+   */
+
+  NSEnumerationReverse = (1UL << 1) /** Specifies that the enumeration should
+   * happen in the opposite of the natural order of the collection.
+   */
+};
+
+/** Bitfield used to specify options to control enumeration over collections.
+ */
+typedef NSUInteger NSEnumerationOptions;
+
+#import <GNUstepBase/GSObjCRuntime.h>
 
 #if OS_API_VERSION(100500,GS_API_LATEST) 
 GS_EXPORT NSString	*NSStringFromProtocol(Protocol *aProtocol);
@@ -50,7 +104,7 @@ GS_EXPORT SEL		NSSelectorFromString(NSString *aSelectorName);
 GS_EXPORT Class		NSClassFromString(NSString *aClassName);
 GS_EXPORT NSString	*NSStringFromClass(Class aClass);
 GS_EXPORT const char	*NSGetSizeAndAlignment(const char *typePtr,
-  unsigned int *sizep, unsigned int *alignp);
+  NSUInteger *sizep, NSUInteger *alignp);
 
 #if OS_API_VERSION(GS_API_NONE, GS_API_NONE)
 /* Logging */
@@ -79,6 +133,19 @@ GS_EXPORT void			NSLogv (NSString *format, va_list args);
 #define nil		0
 #endif
 
+/**
+ * Contains values <code>NSOrderedSame</code>, <code>NSOrderedAscending</code>
+ * <code>NSOrderedDescending</code>, for left hand side equals, less than, or
+ * greater than right hand side.
+ */
+typedef enum _NSComparisonResult 
+{
+  NSOrderedAscending = -1, NSOrderedSame, NSOrderedDescending
+} 
+NSComparisonResult;
+
+enum {NSNotFound = NSIntegerMax};
+
 #if	defined(__cplusplus)
 }
 #endif
